<?php 

/**

 * The core plugin class.

 *

 * This is used to define internationalization, admin-specific hooks, and

 * public-facing site hooks.

 *

 * Also maintains the unique identifier of this plugin as well as the current

 * version of the plugin.

 *

 * @since      1.0.0

 * @package    WC_PayVyne_Payment_Gateway

 * @subpackage WC_PayVyne_Payment_Gateway/includes/

 */

class WC_PayVyne_Payment_Gateway extends WC_Payment_Gateway{



    private $order_status;



    CONST SANDBOX_AUTH_URL = "https://uat.app.payvyne.com/api/oauth/token?grant_type=client_credentials";



    CONST AUTH_URL = "https://app.payvyne.com/api/oauth/token?grant_type=client_credentials";



    CONST SANDBOX_GATEWAY_URL = "https://uat.app.payvyne.com/api/v1/payments";



    CONST GATEWAY_URL = "https://app.payvyne.com/api/v1/payments";



    CONST LOGFILE = 'payvyne.log';



    public function __construct(){

    	$this->id = 'payvyne_payment';

    	if($this->get_option('display_image') == "no"){

    	  $this->icon                 = null;

    	}else{

    	  $this->icon                 = $this->get_option('payment_image') ? $this->get_option('payment_image') : $this->getbase64image();

    	}

    	$this->has_fields           = false;

    	$this->method_title         = $this->method_title ? $this->method_title : __('PayVyne Payment','payvyne');

    	$this->method_description   = __('Pay Via PayVyne','payvyne');



        // Define user set variables.

        $this->title            = $this->get_option( 'title' );

        $this->description      = $this->get_option( 'description' );

        $this->redirect_type    = $this->get_option( 'redirect_type' ) ? $this->get_option( 'redirect_type' ) : "URL";

        $this->testmode         = $this->get_option( 'test_mode' );

        $this->debug            = $this->get_option( 'log_mode' );

        $this->order_status     = $this->get_option( 'order_status' );

        



        // Load Resources

        $this->load_payvyne_resources();



    	// Load the settings.

    	$this->init_form_fields();

    	$this->init_settings();



    	if($this->testmode == 'yes'){

    		$this->client_id        = $this->get_option( 'sandbox_client_id' );

    		$this->client_secret  	= $this->get_option( 'sandbox_client_secret' );

    		$this->auth_endpoint  	= self::SANDBOX_AUTH_URL;

    		$this->live_endpoint  	= self::SANDBOX_GATEWAY_URL;

            $this->bank_account     = $this->get_option( 'sandbox_bank_account' );

    	}else{

    		$this->client_id        = $this->get_option( 'client_id' );

    		$this->client_secret  	= $this->get_option( 'client_secret' );

    		$this->auth_endpoint  	= self::AUTH_URL;

    		$this->live_endpoint  	= self::GATEWAY_URL;

            $this->bank_account     = $this->get_option( 'bank_account' );

    	}



    	add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

    }



    public function load_payvyne_resources(){

    	wp_register_style('payvyne.main', plugins_url('css/payvyne.css' , __FILE__ ));

    	wp_register_style('payvyne.admin', plugins_url('css/payvyne.admin.css' , __FILE__ ));

		wp_register_script('payvyne.admin.js', plugins_url('js/payvyne.admin.js', __FILE__ ), array('jquery'));

    	wp_localize_script( 'payvyne.admin.js', 'payvyne', array( 

            'plugin_url' => plugin_dir_url( __DIR__ ),

            'security_key' => md5('payvyne')

        ));

    }



    public function init_form_fields(){

    	$this->form_fields = include("payvyne-settings.php");

    }



    /**

     * Include Custom Script.

     *

     */

    public function generate_custom_script_html() {

        wp_enqueue_script('payvyne.admin.js');

    }



    public function redirect_to_checkout(){

        // Notify the user that payment is failed. 

        wc_add_notice( sprintf( '<span class="payvyne_fails">' .

                    __( 'Sorry, The transaction has failed from the PayVyne.', 'payvyne' ) .'</span>'

        ), 'error' );



        // if its fails

        return array(

          'result' => 'error', 

          'redirect' => wc_get_checkout_url()

        );

    }



     /**

     * Process payment.

     *

     */

	public function process_payment( $order_id ) {

		global $woocommerce;

		$order = new WC_Order( $order_id );

        $this::Log("=========================");

        $this::Log("Init PayVyne");

		// Mark as on-hold (we're awaiting the cheque)

		$order->update_status($this->order_status, __( 'Awaiting payment', 'payvyne' ));



        $this::Log("PayVyne - Auth Call");

        // added the auth details need to pass to the body parameter make sure that

        $auth = array(

            'client_id' => $this->client_id,

            'client_secret' => $this->client_secret

        );



        $result = wp_remote_post($this->auth_endpoint, array(

            'body' => $auth

        ));



        $this::Log("PayVyne - Auth Result");

        $this::Log(print_r($result['body'], true));



        try {

            // If we got the access token then proceed otherwise prevent the order

            if($result['response']['code'] == 200){

                $this::Log("Getting Access Token");

                $data = json_decode($result['body']);

                $this->access_token = $data->access_token;

            }else{

                $this::Log("Auth Call Fail");

                $this::Log("Ending PayVyne");

                $this::Log("=========================");

                return $this->redirect_to_checkout();

            }

        } catch (Exception $e) {

            $this::Log("Unknown error occurred");

            $this::Log(print_r($e->getMessage(), true));

            $this::Log("Ending PayVyne");

            $this::Log("=========================");

            return $this->redirect_to_checkout();

        }



        // Request payload

        $payload = array(

            'amount' => $order->get_total(),

            'currency' => get_woocommerce_currency(),

            'description' => 'Order from '.get_bloginfo().' Order ID: '.$order_id,

            'callbackUrl' => site_url('payvyne/'. $order_id),

            'mediaType' => $this->redirect_type,

            'destinationAccount' => $this->bank_account,

        );



        $result = wp_remote_post($this->live_endpoint, array(

            'body'    => wp_json_encode($payload),

            'headers' => array(

                'Authorization' => 'Bearer '. $this->access_token,

                'content-type' => 'application/json',

            ),

        ));



        $this::Log("PayVyne - Gateway Call Result");

        $this::Log(print_r($result['body'], true));



        try {

            // If we got the access token then proceed otherwise prevent the order

            if($result['response']['code'] == 200){

                $this::Log("Redirecting to the gateway..");

                $data = json_decode($result['body']);



                $note = __("Redirected to the PayVyne with Request ID: ".$data->requestId, 'payvyne');

                $order->add_order_note( $note );

                $this::Log("Adding notes to the Order..");



                // Return thankyou redirect

                return array(

                    'result' => 'success',

                    'redirect' => $data->redirectUrl

                );  



            }else{

                $this::Log("Payment fail");

                $this::Log("Ending PayVyne");

                $this::Log("=========================");

                return $this->redirect_to_checkout();

            }

        } catch (Exception $e) {

            $this::Log("Unknown error occurred");

            $this::Log(print_r($e->getMessage(), true));

            $this::Log("Ending PayVyne");

            $this::Log("=========================");

            return $this->redirect_to_checkout();

        }		

	}



    /**

    * Base65 image.

    * @return string

    *

    */

	public function getbase64image(){

		$logo = include("payvyne-logo.php");

		return $logo; 

	}



    /* Logger Function */

    public static function Log($str) {

        $setting = get_option('woocommerce_payvyne_payment_settings');

        if(isset($setting['log_mode']) && $setting['log_mode'] == "yes"){

            $dirpath = dirname( __FILE__ );

            $root = substr($dirpath, 0, strpos($dirpath, "wp-content"));

            $file = $root.'log'.DIRECTORY_SEPARATOR.self::LOGFILE;

            $dirname = dirname($file);



            if (!is_dir($dirname)){

                mkdir($dirname, 0755, true);

            }



            if(!file_exists($file)){

                fopen($file,'a+');

            }



            $d = date("j-M-Y H:i:s e");

            error_log("[$d] $str \n", 3, $file);

        }

        

        return true;

    }

}

