<?php
/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    WC_PayVyne_Payment_Gateway
 * @subpackage WC_PayVyne_Payment_Gateway/includes/
 */
class WC_Payvyne_Payment_Redirect_Handler {

    CONST SANDBOX_STATUS_UPDATE_URL = 'https://uat.app.payvyne.com/api/v1/payments/';

    CONST STATUS_UPDATE_URL = 'https://app.payvyne.com/api/v1/payments/';

    public function __construct(){
        $this->exceute_actions();
    }

    public function exceute_actions(){
        add_action( 'init', array( $this, 'payvyne_add_calback' ) );
        add_action( 'pre_get_posts', array( $this, 'payvyne_handle_callback' ) );
    }

    public function payvyne_add_calback(){
        add_rewrite_endpoint( 'payvyne', EP_ROOT );
    }

    public function UpdateOrder($data, $order_id){
        if(isset($data['paymentStatus'])
            && ( $data['paymentStatus'] == "NO_CONSENT"
            || $data['paymentStatus'] == "FAILED"
            || $data['paymentStatus'] == "EXPIRED"
            || $data['paymentStatus'] == "NO_CONFIRM")
        ){
            $order = new WC_Order( $order_id );
            $order->update_status( 'failed' );

        }else if(isset($data['paymentStatus'])
                && ( $data['paymentStatus'] == "INITIATED"
                || $data['paymentStatus'] == "SENT"
                || $data['paymentStatus'] == "COMPLETED"
                || $data['paymentStatus'] == "SETTLED")
            ) {

            $order = new WC_Order( $order_id );
            $order->update_status( 'processing' );
        }

        return true;
    }

    public function payvyne_handle_callback( $query ){
        if ( $query->is_main_query() && isset($query->query['payvyne'])) {
            WC_PayVyne_Payment_Gateway::Log("=========================");
            WC_PayVyne_Payment_Gateway::Log("Back to Site");
            if(isset($_POST)){
                $post = file_get_contents("php://input");
                $result = json_decode($post);
                WC_PayVyne_Payment_Gateway::Log("Auto Updates - Received Updated Status: ". $result->status);
                if($result->type == "PAYMENT_STATUS_CHANGE"){
                    $paymentId = $result->paymentId;
                    $order_id = $query->query['payvyne'];
                    //$orders = get_posts(array('post_type' => 'shop_order'));
                    $orders = wc_get_orders( array('numberposts' => -1) );
                    foreach ($orders as $order) {
                        $order_id = $order->ID;
                        $order = new WC_Order( $order_id );
                        $transaction_id = $order->get_transaction_id();
                        if($transaction_id == $paymentId){
                            WC_PayVyne_Payment_Gateway::Log("Auto Updates - Received Updateeed Status: ". $result->status);
                            $final_res['paymentStatus'] = $result->status;
                            //$final_res['paymentId'] = $updates->id;
                            $this->UpdateOrder($final_res, $order_id);
                            die;
                        }
                    }
                }
            }


            $result = base64_decode($_GET['payvyne_payment_payload']);
            $result = explode("}",$result, 3);

            global $woocommerce;
            $order_id = $query->query['payvyne'];
            $order = new WC_Order( $order_id );

            $final_res = array();

            if(isset($result[1])){
                $final_res = json_decode($result[1]."}", true);
                WC_PayVyne_Payment_Gateway::Log("Status Received: ".$final_res['paymentStatus']);
            }
            //WC_PayVyne_Payment_Gateway::Log("==========".$final_res['paymentStatus']."===============");

            if(isset($final_res['paymentStatus'])
                && $final_res['paymentStatus'] == "PROCESSING"
            ){
                WC_PayVyne_Payment_Gateway::Log("Checking if any status updates..");
                $updates = $this->UpdateStatus($final_res['paymentId']);
                if(!empty($updates)){
                    WC_PayVyne_Payment_Gateway::Log("Received Updated Status: ". $updates->status);
                    $final_res['paymentStatus'] = $updates->status;
                    $final_res['paymentId'] = $updates->id;
                }
            }

            if(isset($final_res['paymentStatus'])
                && ( $final_res['paymentStatus'] == "NO_CONSENT"
                || $final_res['paymentStatus'] == "FAILED"
                || $final_res['paymentStatus'] == "EXPIRED"
                || $final_res['paymentStatus'] == "NO_CONFIRM")
            ){
                // if Payment fails
                $order->update_status( 'failed' );

                // Notify the user that payment is failed.
                wc_add_notice( sprintf( '<span class="payvyne_fails">' .
                    __( 'Sorry, The transaction has failed from the PayVyne.', 'payvyne' ) .'</span>'
                ), 'error' );

                update_post_meta( $order_id, '_transaction_id', $final_res['paymentId'] );
                $redirect = wc_get_checkout_url();
                WC_PayVyne_Payment_Gateway::Log("=========================");
                wp_redirect( $redirect );
                die;
            }else if(isset($final_res['paymentStatus'])
                && ( $final_res['paymentStatus'] == "INITIATED"
                || $final_res['paymentStatus'] == "SENT"
                || $final_res['paymentStatus'] == "COMPLETED"
                || $final_res['paymentStatus'] == "PROCESSING"
                || $final_res['paymentStatus'] == "SETTLED")
            ) {
                // if payment is successful..
                if($final_res['paymentStatus'] == "PROCESSING"){
                    $order->update_status( 'pending payment' );
                }else{
                    $order->update_status( 'processing' );
                }
                $redirecturl = $order->get_checkout_order_received_url();
                update_post_meta( $order_id, '_transaction_id', $final_res['paymentId'] );

                // Reduce stock levels
                wc_reduce_stock_levels( $order_id );

                // Remove cart
                $woocommerce->cart->empty_cart();
                WC_PayVyne_Payment_Gateway::Log("=========================");
                wp_redirect( $redirecturl );
                die;
            }else{
                // Notify the user that payment is failed.
                wc_add_notice( sprintf( '<span class="payvyne_fails">' .
                    __( 'Sorry, The transaction has failed from the PayVyne. To know more click ', 'payvyne' ) . '<a href="#">'.__('here', 'payvyne') .'</a></span>'
                ), 'error' );
                // if Payment fails
                $order->update_status( 'cancelled' );
                WC_PayVyne_Payment_Gateway::Log("=========================");
                $redirect = wc_get_checkout_url();
                wp_redirect( $redirect );
                die;
            }
        }
    }

    public function UpdateStatus($paymentId){
        $setting = get_option('woocommerce_payvyne_payment_settings');
        if($setting['test_mode'] == "yes"){
            $this->client_id        = $setting['sandbox_client_id'];
            $this->client_secret  	= $setting['sandbox_client_secret'];
            $this->auth_endpoint  	= WC_PayVyne_Payment_Gateway::SANDBOX_AUTH_URL;
            $this->live_endpoint  	= self::SANDBOX_STATUS_UPDATE_URL;
        }else{
            $this->client_id        = $setting['client_id'];
            $this->client_secret  	= $setting['client_secret'];
            $this->auth_endpoint  	= WC_PayVyne_Payment_Gateway::AUTH_URL;
            $this->live_endpoint  	= self::STATUS_UPDATE_URL;
        }

        $auth = array(
            'client_id' => $this->client_id,
            'client_secret' => $this->client_secret
        );

        $result = wp_remote_post($this->auth_endpoint, array(
            'body' => $auth
        ));

        try {
            // If we got the access token then proceed otherwise prevent the order
            if($result['response']['code'] == 200){
                WC_PayVyne_Payment_Gateway::Log("Getting Access Token For Update Status");
                $data = json_decode($result['body']);
                $this->access_token = $data->access_token;
            }else{
                WC_PayVyne_Payment_Gateway::Log("Auth Call Fail");
                WC_PayVyne_Payment_Gateway::Log("Ending PayVyne");
                WC_PayVyne_Payment_Gateway::Log("=========================");
                return true;
            }
        } catch (Exception $e) {
            WC_PayVyne_Payment_Gateway::Log("Unknown error occurred");
            WC_PayVyne_Payment_Gateway::Log(print_r($e->getMessage(), true));
            WC_PayVyne_Payment_Gateway::Log("Ending PayVyne");
            WC_PayVyne_Payment_Gateway::Log("=========================");
            return true;
        }


        $result = wp_remote_get($this->live_endpoint.$paymentId, array(
            'headers' => array(
                'Authorization' => 'Bearer '. $this->access_token,
                'content-type' => 'application/json',
            ),
        ));

        try {
            // If we got the access token then proceed otherwise prevent the order
            if($result['response']['code'] == 200){
                WC_PayVyne_Payment_Gateway::Log("Getting Responce..");
                $data = json_decode($result['body']);
            }else{
                WC_PayVyne_Payment_Gateway::Log("Status Update Call Fail");
                WC_PayVyne_Payment_Gateway::Log("Ending PayVyne");
                WC_PayVyne_Payment_Gateway::Log("=========================");
                return true;
            }
        } catch (Exception $e) {
            WC_PayVyne_Payment_Gateway::Log("Unknown error occurred");
            WC_PayVyne_Payment_Gateway::Log(print_r($e->getMessage(), true));
            WC_PayVyne_Payment_Gateway::Log("Ending PayVyne");
            WC_PayVyne_Payment_Gateway::Log("=========================");
            return true;
        }

        return $data;
    }
}
