<?php

/**

 * The plugin file

 *

 * This file is read by WordPress to generate the plugin information in the plugin

 * admin area. This file also includes all of the dependencies used by the plugin,

 * registers the activation and deactivation functions, and defines a function

 * that starts the plugin.

 *

 * @wordpress-plugin

 * Plugin Name:     payVyne

 * Description:     Fast, transparent and secure Open Banking powered account to account payments - the best way for you to pay and get paid.

 * Version:         1.0.2

 * License:         GPL-2.0+

 * License URI:     http://www.gnu.org/licenses/gpl-2.0.txt

 * Text Domain:     payvyne

 * Domain Path:     /languages

 */



// If this file is called directly, abort.

defined('WPINC') || die;



/**

 * Currently plugin version.

 * Start at version 1.0.0 and use SemVer - https://semver.org

 * Rename this for your plugin and update it as you release new versions.

 */

define('PAYVYNE_FOR_WOOCOMMERCE_VERSION', '1.0.2');



/**

 * The full path and filename of the this file.

 */

define('PAYVYNE_FOR_WOOCOMMERCE_FILE', __FILE__);



/**

 * The core plugin class that is used to define internationalization,

 * admin-specific hooks, and public-facing site hooks.

 */

$active_plugins = apply_filters('active_plugins', get_option('active_plugins'));



if(payvyne_payment_is_woocommerce_active()){

	add_filter('woocommerce_payment_gateways', 'add_payvyne_payment_gateway');

	function add_payvyne_payment_gateway( $gateways ){

		$gateways[] = 'WC_PayVyne_Payment_Gateway';

		return $gateways;

	}



	add_action('plugins_loaded', 'init_payvyne_payment_gateway');

	function init_payvyne_payment_gateway(){

		flush_rewrite_rules();

		require plugin_dir_path( __FILE__ ) .'includes'. DIRECTORY_SEPARATOR .'class-payvyne-gateways-for-woocommerce.php';



		/**

		 * Redirect handler for Payment Success and fail senario

		 */

		require plugin_dir_path( __FILE__ ) .'includes'. DIRECTORY_SEPARATOR .'class-payvyne-gateways-redirect-handler-for-woocommerce.php';



		new WC_Payvyne_Payment_Redirect_Handler();

	}



	add_action( 'plugins_loaded', 'payvyne_payment_load_plugin_textdomain' );

	function payvyne_payment_load_plugin_textdomain() {

	  load_plugin_textdomain( 'payvyne', FALSE, basename( dirname( __FILE__ ) ) . '/languages/' );

	}

}



/**

 * @return bool

 */

function payvyne_payment_is_woocommerce_active()

{

	$active_plugins = (array) get_option('active_plugins', array());



	if (is_multisite()) {

		$active_plugins = array_merge($active_plugins, get_site_option('active_sitewide_plugins', array()));

	}



	return in_array('woocommerce/woocommerce.php', $active_plugins) || array_key_exists('woocommerce/woocommerce.php', $active_plugins);

}
